# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/commands.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import (
    duration_pb2 as google_dot_protobuf_dot_duration__pb2,
    timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2,
)

from . import value_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/commands.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\022CommandsOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n%com/daml/ledger/api/v1/commands.proto\x12\x16\x63om.daml.ledger.api.v1\x1a"com/daml/ledger/api/v1/value.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto"\xf5\x02\n\x08\x43ommands\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x13\n\x0bworkflow_id\x18\x02 \x01(\t\x12\x16\n\x0e\x61pplication_id\x18\x03 \x01(\t\x12\x12\n\ncommand_id\x18\x04 \x01(\t\x12\r\n\x05party\x18\x05 \x01(\t\x12\x31\n\x08\x63ommands\x18\x08 \x03(\x0b\x32\x1f.com.daml.ledger.api.v1.Command\x12\x35\n\x12\x64\x65\x64uplication_time\x18\t \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\x13min_ledger_time_abs\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x13min_ledger_time_rel\x18\x0b \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x0e\n\x06\x61\x63t_as\x18\x0c \x03(\t\x12\x0f\n\x07read_as\x18\r \x03(\tJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08"\xa0\x02\n\x07\x43ommand\x12\x37\n\x06\x63reate\x18\x01 \x01(\x0b\x32%.com.daml.ledger.api.v1.CreateCommandH\x00\x12;\n\x08\x65xercise\x18\x02 \x01(\x0b\x32\'.com.daml.ledger.api.v1.ExerciseCommandH\x00\x12\x45\n\rexerciseByKey\x18\x04 \x01(\x0b\x32,.com.daml.ledger.api.v1.ExerciseByKeyCommandH\x00\x12M\n\x11\x63reateAndExercise\x18\x03 \x01(\x0b\x32\x30.com.daml.ledger.api.v1.CreateAndExerciseCommandH\x00\x42\t\n\x07\x63ommand"\x82\x01\n\rCreateCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32".com.daml.ledger.api.v1.Identifier\x12\x38\n\x10\x63reate_arguments\x18\x02 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.Record"\xa7\x01\n\x0f\x45xerciseCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32".com.daml.ledger.api.v1.Identifier\x12\x13\n\x0b\x63ontract_id\x18\x02 \x01(\t\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value"\xcc\x01\n\x14\x45xerciseByKeyCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32".com.daml.ledger.api.v1.Identifier\x12\x33\n\x0c\x63ontract_key\x18\x02 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.Value"\xd5\x01\n\x18\x43reateAndExerciseCommand\x12\x37\n\x0btemplate_id\x18\x01 \x01(\x0b\x32".com.daml.ledger.api.v1.Identifier\x12\x38\n\x10\x63reate_arguments\x18\x02 \x01(\x0b\x32\x1e.com.daml.ledger.api.v1.Record\x12\x0e\n\x06\x63hoice\x18\x03 \x01(\t\x12\x36\n\x0f\x63hoice_argument\x18\x04 \x01(\x0b\x32\x1d.com.daml.ledger.api.v1.ValueBE\n\x16\x63om.daml.ledger.api.v1B\x12\x43ommandsOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,
    ],
)


_COMMANDS = _descriptor.Descriptor(
    name="Commands",
    full_name="com.daml.ledger.api.v1.Commands",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.Commands.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="workflow_id",
            full_name="com.daml.ledger.api.v1.Commands.workflow_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="application_id",
            full_name="com.daml.ledger.api.v1.Commands.application_id",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="command_id",
            full_name="com.daml.ledger.api.v1.Commands.command_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="party",
            full_name="com.daml.ledger.api.v1.Commands.party",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="commands",
            full_name="com.daml.ledger.api.v1.Commands.commands",
            index=5,
            number=8,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="deduplication_time",
            full_name="com.daml.ledger.api.v1.Commands.deduplication_time",
            index=6,
            number=9,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="min_ledger_time_abs",
            full_name="com.daml.ledger.api.v1.Commands.min_ledger_time_abs",
            index=7,
            number=10,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="min_ledger_time_rel",
            full_name="com.daml.ledger.api.v1.Commands.min_ledger_time_rel",
            index=8,
            number=11,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="act_as",
            full_name="com.daml.ledger.api.v1.Commands.act_as",
            index=9,
            number=12,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="read_as",
            full_name="com.daml.ledger.api.v1.Commands.read_as",
            index=10,
            number=13,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=167,
    serialized_end=540,
)


_COMMAND = _descriptor.Descriptor(
    name="Command",
    full_name="com.daml.ledger.api.v1.Command",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="create",
            full_name="com.daml.ledger.api.v1.Command.create",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exercise",
            full_name="com.daml.ledger.api.v1.Command.exercise",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exerciseByKey",
            full_name="com.daml.ledger.api.v1.Command.exerciseByKey",
            index=2,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="createAndExercise",
            full_name="com.daml.ledger.api.v1.Command.createAndExercise",
            index=3,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="command",
            full_name="com.daml.ledger.api.v1.Command.command",
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=543,
    serialized_end=831,
)


_CREATECOMMAND = _descriptor.Descriptor(
    name="CreateCommand",
    full_name="com.daml.ledger.api.v1.CreateCommand",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="template_id",
            full_name="com.daml.ledger.api.v1.CreateCommand.template_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="create_arguments",
            full_name="com.daml.ledger.api.v1.CreateCommand.create_arguments",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=834,
    serialized_end=964,
)


_EXERCISECOMMAND = _descriptor.Descriptor(
    name="ExerciseCommand",
    full_name="com.daml.ledger.api.v1.ExerciseCommand",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="template_id",
            full_name="com.daml.ledger.api.v1.ExerciseCommand.template_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contract_id",
            full_name="com.daml.ledger.api.v1.ExerciseCommand.contract_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice",
            full_name="com.daml.ledger.api.v1.ExerciseCommand.choice",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice_argument",
            full_name="com.daml.ledger.api.v1.ExerciseCommand.choice_argument",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=967,
    serialized_end=1134,
)


_EXERCISEBYKEYCOMMAND = _descriptor.Descriptor(
    name="ExerciseByKeyCommand",
    full_name="com.daml.ledger.api.v1.ExerciseByKeyCommand",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="template_id",
            full_name="com.daml.ledger.api.v1.ExerciseByKeyCommand.template_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="contract_key",
            full_name="com.daml.ledger.api.v1.ExerciseByKeyCommand.contract_key",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice",
            full_name="com.daml.ledger.api.v1.ExerciseByKeyCommand.choice",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice_argument",
            full_name="com.daml.ledger.api.v1.ExerciseByKeyCommand.choice_argument",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1137,
    serialized_end=1341,
)


_CREATEANDEXERCISECOMMAND = _descriptor.Descriptor(
    name="CreateAndExerciseCommand",
    full_name="com.daml.ledger.api.v1.CreateAndExerciseCommand",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="template_id",
            full_name="com.daml.ledger.api.v1.CreateAndExerciseCommand.template_id",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="create_arguments",
            full_name="com.daml.ledger.api.v1.CreateAndExerciseCommand.create_arguments",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice",
            full_name="com.daml.ledger.api.v1.CreateAndExerciseCommand.choice",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="choice_argument",
            full_name="com.daml.ledger.api.v1.CreateAndExerciseCommand.choice_argument",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1344,
    serialized_end=1557,
)

_COMMANDS.fields_by_name["commands"].message_type = _COMMAND
_COMMANDS.fields_by_name[
    "deduplication_time"
].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_COMMANDS.fields_by_name[
    "min_ledger_time_abs"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_COMMANDS.fields_by_name[
    "min_ledger_time_rel"
].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_COMMAND.fields_by_name["create"].message_type = _CREATECOMMAND
_COMMAND.fields_by_name["exercise"].message_type = _EXERCISECOMMAND
_COMMAND.fields_by_name["exerciseByKey"].message_type = _EXERCISEBYKEYCOMMAND
_COMMAND.fields_by_name["createAndExercise"].message_type = _CREATEANDEXERCISECOMMAND
_COMMAND.oneofs_by_name["command"].fields.append(_COMMAND.fields_by_name["create"])
_COMMAND.fields_by_name["create"].containing_oneof = _COMMAND.oneofs_by_name["command"]
_COMMAND.oneofs_by_name["command"].fields.append(_COMMAND.fields_by_name["exercise"])
_COMMAND.fields_by_name["exercise"].containing_oneof = _COMMAND.oneofs_by_name["command"]
_COMMAND.oneofs_by_name["command"].fields.append(_COMMAND.fields_by_name["exerciseByKey"])
_COMMAND.fields_by_name["exerciseByKey"].containing_oneof = _COMMAND.oneofs_by_name["command"]
_COMMAND.oneofs_by_name["command"].fields.append(_COMMAND.fields_by_name["createAndExercise"])
_COMMAND.fields_by_name["createAndExercise"].containing_oneof = _COMMAND.oneofs_by_name["command"]
_CREATECOMMAND.fields_by_name[
    "template_id"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_CREATECOMMAND.fields_by_name[
    "create_arguments"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._RECORD
_EXERCISECOMMAND.fields_by_name[
    "template_id"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISECOMMAND.fields_by_name[
    "choice_argument"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_EXERCISEBYKEYCOMMAND.fields_by_name[
    "template_id"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_EXERCISEBYKEYCOMMAND.fields_by_name[
    "contract_key"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_EXERCISEBYKEYCOMMAND.fields_by_name[
    "choice_argument"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
_CREATEANDEXERCISECOMMAND.fields_by_name[
    "template_id"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._IDENTIFIER
_CREATEANDEXERCISECOMMAND.fields_by_name[
    "create_arguments"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._RECORD
_CREATEANDEXERCISECOMMAND.fields_by_name[
    "choice_argument"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_value__pb2._VALUE
DESCRIPTOR.message_types_by_name["Commands"] = _COMMANDS
DESCRIPTOR.message_types_by_name["Command"] = _COMMAND
DESCRIPTOR.message_types_by_name["CreateCommand"] = _CREATECOMMAND
DESCRIPTOR.message_types_by_name["ExerciseCommand"] = _EXERCISECOMMAND
DESCRIPTOR.message_types_by_name["ExerciseByKeyCommand"] = _EXERCISEBYKEYCOMMAND
DESCRIPTOR.message_types_by_name["CreateAndExerciseCommand"] = _CREATEANDEXERCISECOMMAND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Commands = _reflection.GeneratedProtocolMessageType(
    "Commands",
    (_message.Message,),
    {
        "DESCRIPTOR": _COMMANDS,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Commands)
    },
)
_sym_db.RegisterMessage(Commands)

Command = _reflection.GeneratedProtocolMessageType(
    "Command",
    (_message.Message,),
    {
        "DESCRIPTOR": _COMMAND,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Command)
    },
)
_sym_db.RegisterMessage(Command)

CreateCommand = _reflection.GeneratedProtocolMessageType(
    "CreateCommand",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATECOMMAND,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreateCommand)
    },
)
_sym_db.RegisterMessage(CreateCommand)

ExerciseCommand = _reflection.GeneratedProtocolMessageType(
    "ExerciseCommand",
    (_message.Message,),
    {
        "DESCRIPTOR": _EXERCISECOMMAND,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExerciseCommand)
    },
)
_sym_db.RegisterMessage(ExerciseCommand)

ExerciseByKeyCommand = _reflection.GeneratedProtocolMessageType(
    "ExerciseByKeyCommand",
    (_message.Message,),
    {
        "DESCRIPTOR": _EXERCISEBYKEYCOMMAND,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ExerciseByKeyCommand)
    },
)
_sym_db.RegisterMessage(ExerciseByKeyCommand)

CreateAndExerciseCommand = _reflection.GeneratedProtocolMessageType(
    "CreateAndExerciseCommand",
    (_message.Message,),
    {
        "DESCRIPTOR": _CREATEANDEXERCISECOMMAND,
        "__module__": "com.daml.ledger.api.v1.commands_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.CreateAndExerciseCommand)
    },
)
_sym_db.RegisterMessage(CreateAndExerciseCommand)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2

from . import (
    event_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2,
    trace_context_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/transaction.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\025TransactionOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n(com/daml/ledger/api/v1/transaction.proto\x12\x16\x63om.daml.ledger.api.v1\x1a"com/daml/ledger/api/v1/event.proto\x1a*com/daml/ledger/api/v1/trace_context.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x8f\x03\n\x0fTransactionTree\x12\x16\n\x0etransaction_id\x18\x01 \x01(\t\x12\x12\n\ncommand_id\x18\x02 \x01(\t\x12\x13\n\x0bworkflow_id\x18\x03 \x01(\t\x12\x30\n\x0c\x65\x66\x66\x65\x63tive_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06offset\x18\x06 \x01(\t\x12M\n\x0c\x65vents_by_id\x18\x07 \x03(\x0b\x32\x37.com.daml.ledger.api.v1.TransactionTree.EventsByIdEntry\x12\x16\n\x0eroot_event_ids\x18\x08 \x03(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext\x1aT\n\x0f\x45ventsByIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.com.daml.ledger.api.v1.TreeEvent:\x02\x38\x01"\x89\x01\n\tTreeEvent\x12\x37\n\x07\x63reated\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.CreatedEventH\x00\x12;\n\texercised\x18\x02 \x01(\x0b\x32&.com.daml.ledger.api.v1.ExercisedEventH\x00\x42\x06\n\x04kind"\xfd\x01\n\x0bTransaction\x12\x16\n\x0etransaction_id\x18\x01 \x01(\t\x12\x12\n\ncommand_id\x18\x02 \x01(\t\x12\x13\n\x0bworkflow_id\x18\x03 \x01(\t\x12\x30\n\x0c\x65\x66\x66\x65\x63tive_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\x06\x65vents\x18\x05 \x03(\x0b\x32\x1d.com.daml.ledger.api.v1.Event\x12\x0e\n\x06offset\x18\x06 \x01(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContextBH\n\x16\x63om.daml.ledger.api.v1B\x15TransactionOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2.DESCRIPTOR,
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
    ],
)


_TRANSACTIONTREE_EVENTSBYIDENTRY = _descriptor.Descriptor(
    name="EventsByIdEntry",
    full_name="com.daml.ledger.api.v1.TransactionTree.EventsByIdEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="com.daml.ledger.api.v1.TransactionTree.EventsByIdEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="com.daml.ledger.api.v1.TransactionTree.EventsByIdEntry.value",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b"8\001",
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=497,
    serialized_end=581,
)

_TRANSACTIONTREE = _descriptor.Descriptor(
    name="TransactionTree",
    full_name="com.daml.ledger.api.v1.TransactionTree",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transaction_id",
            full_name="com.daml.ledger.api.v1.TransactionTree.transaction_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="command_id",
            full_name="com.daml.ledger.api.v1.TransactionTree.command_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="workflow_id",
            full_name="com.daml.ledger.api.v1.TransactionTree.workflow_id",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="effective_at",
            full_name="com.daml.ledger.api.v1.TransactionTree.effective_at",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="offset",
            full_name="com.daml.ledger.api.v1.TransactionTree.offset",
            index=4,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="events_by_id",
            full_name="com.daml.ledger.api.v1.TransactionTree.events_by_id",
            index=5,
            number=7,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="root_event_ids",
            full_name="com.daml.ledger.api.v1.TransactionTree.root_event_ids",
            index=6,
            number=8,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.TransactionTree.trace_context",
            index=7,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[
        _TRANSACTIONTREE_EVENTSBYIDENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=182,
    serialized_end=581,
)


_TREEEVENT = _descriptor.Descriptor(
    name="TreeEvent",
    full_name="com.daml.ledger.api.v1.TreeEvent",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="created",
            full_name="com.daml.ledger.api.v1.TreeEvent.created",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exercised",
            full_name="com.daml.ledger.api.v1.TreeEvent.exercised",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="kind",
            full_name="com.daml.ledger.api.v1.TreeEvent.kind",
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=584,
    serialized_end=721,
)


_TRANSACTION = _descriptor.Descriptor(
    name="Transaction",
    full_name="com.daml.ledger.api.v1.Transaction",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transaction_id",
            full_name="com.daml.ledger.api.v1.Transaction.transaction_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="command_id",
            full_name="com.daml.ledger.api.v1.Transaction.command_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="workflow_id",
            full_name="com.daml.ledger.api.v1.Transaction.workflow_id",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="effective_at",
            full_name="com.daml.ledger.api.v1.Transaction.effective_at",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="events",
            full_name="com.daml.ledger.api.v1.Transaction.events",
            index=4,
            number=5,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="offset",
            full_name="com.daml.ledger.api.v1.Transaction.offset",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.Transaction.trace_context",
            index=6,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=724,
    serialized_end=977,
)

_TRANSACTIONTREE_EVENTSBYIDENTRY.fields_by_name["value"].message_type = _TREEEVENT
_TRANSACTIONTREE_EVENTSBYIDENTRY.containing_type = _TRANSACTIONTREE
_TRANSACTIONTREE.fields_by_name[
    "effective_at"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRANSACTIONTREE.fields_by_name["events_by_id"].message_type = _TRANSACTIONTREE_EVENTSBYIDENTRY
_TRANSACTIONTREE.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_TREEEVENT.fields_by_name[
    "created"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2._CREATEDEVENT
_TREEEVENT.fields_by_name[
    "exercised"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2._EXERCISEDEVENT
_TREEEVENT.oneofs_by_name["kind"].fields.append(_TREEEVENT.fields_by_name["created"])
_TREEEVENT.fields_by_name["created"].containing_oneof = _TREEEVENT.oneofs_by_name["kind"]
_TREEEVENT.oneofs_by_name["kind"].fields.append(_TREEEVENT.fields_by_name["exercised"])
_TREEEVENT.fields_by_name["exercised"].containing_oneof = _TREEEVENT.oneofs_by_name["kind"]
_TRANSACTION.fields_by_name[
    "effective_at"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TRANSACTION.fields_by_name[
    "events"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_event__pb2._EVENT
_TRANSACTION.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
DESCRIPTOR.message_types_by_name["TransactionTree"] = _TRANSACTIONTREE
DESCRIPTOR.message_types_by_name["TreeEvent"] = _TREEEVENT
DESCRIPTOR.message_types_by_name["Transaction"] = _TRANSACTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TransactionTree = _reflection.GeneratedProtocolMessageType(
    "TransactionTree",
    (_message.Message,),
    {
        "EventsByIdEntry": _reflection.GeneratedProtocolMessageType(
            "EventsByIdEntry",
            (_message.Message,),
            {
                "DESCRIPTOR": _TRANSACTIONTREE_EVENTSBYIDENTRY,
                "__module__": "com.daml.ledger.api.v1.transaction_pb2"
                # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.TransactionTree.EventsByIdEntry)
            },
        ),
        "DESCRIPTOR": _TRANSACTIONTREE,
        "__module__": "com.daml.ledger.api.v1.transaction_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.TransactionTree)
    },
)
_sym_db.RegisterMessage(TransactionTree)
_sym_db.RegisterMessage(TransactionTree.EventsByIdEntry)

TreeEvent = _reflection.GeneratedProtocolMessageType(
    "TreeEvent",
    (_message.Message,),
    {
        "DESCRIPTOR": _TREEEVENT,
        "__module__": "com.daml.ledger.api.v1.transaction_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.TreeEvent)
    },
)
_sym_db.RegisterMessage(TreeEvent)

Transaction = _reflection.GeneratedProtocolMessageType(
    "Transaction",
    (_message.Message,),
    {
        "DESCRIPTOR": _TRANSACTION,
        "__module__": "com.daml.ledger.api.v1.transaction_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.Transaction)
    },
)
_sym_db.RegisterMessage(Transaction)


DESCRIPTOR._options = None
_TRANSACTIONTREE_EVENTSBYIDENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""
This module contains the mapping between gRPC calls and Python/dazl types.
"""

import asyncio
from typing import AbstractSet, Any, AsyncIterable, Collection, Mapping, Optional, Sequence, Union
import uuid
import warnings

from grpc.aio import Channel, UnaryStreamCall

from .. import aio
from ..._gen.com.daml.ledger.api.v1.active_contracts_service_pb2 import (
    GetActiveContractsRequest as G_GetActiveContractsRequest,
)
from ..._gen.com.daml.ledger.api.v1.active_contracts_service_pb2_grpc import (
    ActiveContractsServiceStub,
)
from ..._gen.com.daml.ledger.api.v1.admin.package_management_service_pb2 import (
    UploadDarFileRequest as G_UploadDarFileRequest,
)
from ..._gen.com.daml.ledger.api.v1.admin.package_management_service_pb2_grpc import (
    PackageManagementServiceStub,
)
from ..._gen.com.daml.ledger.api.v1.admin.party_management_service_pb2 import (
    AllocatePartyRequest as G_AllocatePartyRequest,
    ListKnownPartiesRequest as G_ListKnownPartiesRequest,
)
from ..._gen.com.daml.ledger.api.v1.admin.party_management_service_pb2_grpc import (
    PartyManagementServiceStub,
)
from ..._gen.com.daml.ledger.api.v1.command_service_pb2 import (
    SubmitAndWaitRequest as G_SubmitAndWaitRequest,
)
from ..._gen.com.daml.ledger.api.v1.command_service_pb2_grpc import CommandServiceStub
from ..._gen.com.daml.ledger.api.v1.commands_pb2 import Command as G_Command, Commands as G_Commands
from ..._gen.com.daml.ledger.api.v1.ledger_identity_service_pb2 import (
    GetLedgerIdentityRequest as G_GetLedgerIdentityRequest,
)
from ..._gen.com.daml.ledger.api.v1.ledger_identity_service_pb2_grpc import (
    LedgerIdentityServiceStub,
)
from ..._gen.com.daml.ledger.api.v1.package_service_pb2 import (
    GetPackageRequest as G_GetPackageRequest,
    ListPackagesRequest as G_ListPackagesRequest,
)
from ..._gen.com.daml.ledger.api.v1.package_service_pb2_grpc import PackageServiceStub
from ..._gen.com.daml.ledger.api.v1.transaction_filter_pb2 import (
    TransactionFilter as G_TransactionFilter,
)
from ..._gen.com.daml.ledger.api.v1.transaction_service_pb2 import (
    GetTransactionsRequest as G_GetTransactionsRequest,
)
from ..._gen.com.daml.ledger.api.v1.transaction_service_pb2_grpc import TransactionServiceStub
from ...damlast.daml_lf_1 import PackageRef, TypeConName
from ...damlast.util import is_match
from ...prim import LEDGER_STRING_REGEX, ContractData, ContractId, Party
from ...query import Filter, Queries, Query, parse_query
from .._offsets import UNTIL_END, LedgerOffsetRange, from_offset_until_forever
from ..api_types import ArchiveEvent, Boundary, Command, CreateEvent, ExerciseResponse, PartyInfo
from ..config import Config
from ..config.access import PropertyBasedAccessConfig
from ..errors import ProtocolWarning
from .channel import create_channel
from .codec_aio import Codec

__all__ = ["Connection", "QueryStream"]


class Connection(aio.Connection):
    """
    An asynchronous (``asyncio``) connection to the Daml gRPC Ledger API.


    """

    def __init__(self, config: Config):
        self._config = config
        self._logger = config.logger
        self._channel = create_channel(config)
        self._codec = Codec(self)

    @property
    def config(self) -> Config:
        return self._config

    @property
    def channel(self) -> Channel:
        """
        Provides access to the underlying gRPC channel.
        """
        return self._channel

    @property
    def codec(self) -> Codec:
        return self._codec

    async def __aenter__(self) -> "Connection":
        await self.open()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.close()

    async def open(self) -> None:
        """
        Does final validation of the token, including possibly fetching the ledger ID if it is not
        yet known.
        """
        if not self._config.access.ledger_id:
            # most calls require a ledger ID; if it wasn't supplied as part of our token or we were
            # never given a token in the first place, fetch the ledger ID from the destination
            stub = LedgerIdentityServiceStub(self._channel)
            response = await stub.GetLedgerIdentity(G_GetLedgerIdentityRequest())
            if isinstance(self._config.access, PropertyBasedAccessConfig):
                self._logger.info("Connected to gRPC Ledger API, ledger ID: %s", response.ledger_id)
                self._config.access.ledger_id = response.ledger_id
            else:
                raise ValueError("when using token-based access, the token must contain ledger ID")

    async def close(self) -> None:
        """
        Close the underlying channel. Once the channel is closed, future command submissions,
        streams in progress, and any future streams will fail.
        """
        await self._channel.close()

    # region Write API

    async def submit(
        self,
        __commands: Union[Command, Sequence[Command]],
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> "None":
        """
        Submit one or more commands to the Ledger API.

        You should generally prefer trying to use :meth:`create`, :meth:`exercise`,
        :meth:`exercise_by_key`, or :meth:`create_and_exercise`, as they are available over both
        the gRPC Ledger API and HTTP JSON API; additionally those methods can provide more
        information about what happened.

        This method can be used to submit multiple disparate commands as a single transaction, but
        if you find yourself needing to do this, you may want to consider moving more of your logic
        into Daml so that only a single command is needed from the outside in order to satisfy your
        use case.
        """
        if __commands is None:
            return
        elif isinstance(__commands, Command):
            __commands = [__commands]

        stub = CommandServiceStub(self.channel)

        commands_pb = await asyncio.gather(*map(self._codec.encode_command, __commands))
        request = G_SubmitAndWaitRequest(
            commands=G_Commands(
                ledger_id=self._config.access.ledger_id,
                application_id=self._config.access.application_name,
                command_id=self._command_id(command_id),
                workflow_id=self._workflow_id(workflow_id),
                party=self._ensure_act_as(),
                commands=commands_pb,
                act_as=self._config.access.act_as,
                read_as=self._config.access.read_only_as,
            )
        )
        await stub.SubmitAndWait(request)

    async def create(
        self,
        __template_id: Union[str, TypeConName],
        __payload: ContractData,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> CreateEvent:
        """
        Create a contract for a given template.

        :param __template_id:
            The template of the contract to be created.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __payload:
            Template arguments for the contract to be created.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The :class:`CreateEvent` that represents the contract that was successfully created.
        """
        stub = CommandServiceStub(self.channel)

        commands = [
            G_Command(create=await self._codec.encode_create_command(__template_id, __payload))
        ]
        request = G_SubmitAndWaitRequest(
            commands=G_Commands(
                ledger_id=self._config.access.ledger_id,
                application_id=self._config.access.application_name,
                command_id=self._command_id(command_id),
                workflow_id=self._workflow_id(workflow_id),
                party=self._ensure_act_as(),
                commands=commands,
                act_as=self._config.access.act_as,
                read_as=self._config.access.read_only_as,
            )
        )
        response = await stub.SubmitAndWaitForTransaction(request)

        return await self._codec.decode_created_event(response.transaction.events[0].created)

    async def exercise(
        self,
        __contract_id: ContractId,
        __choice_name: str,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ExerciseResponse:
        """
        Exercise a choice on a contract identified by its contract ID.

        :param __contract_id:
            The contract ID of the contract to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __choice_name:
            The name of the choice to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __argument:
            The choice arguments. Can be omitted for choices that take no argument.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        stub = CommandServiceStub(self.channel)

        commands = [
            G_Command(
                exercise=await self._codec.encode_exercise_command(
                    __contract_id, __choice_name, __argument
                )
            )
        ]
        request = self._submit_and_wait_request(commands, workflow_id, command_id)
        response = await stub.SubmitAndWaitForTransactionTree(request)

        return await self._codec.decode_exercise_response(response.transaction)

    async def create_and_exercise(
        self,
        __template_id: Union[str, TypeConName],
        __payload: ContractData,
        __choice_name: str,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ExerciseResponse:
        """
        Exercise a choice on a newly-created contract, in a single transaction.

        :param __template_id:
            The template of the contract to be created (positional argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __payload:
            Template arguments for the contract to be created (positional argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __choice_name:
            The name of the choice to exercise (positional argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __argument:
            The choice arguments. Can be omitted for choices that take no argument (positional
            argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        stub = CommandServiceStub(self.channel)

        commands = [
            G_Command(
                createAndExercise=await self._codec.encode_create_and_exercise_command(
                    __template_id, __payload, __choice_name, __argument
                )
            )
        ]
        request = self._submit_and_wait_request(commands, workflow_id, command_id)
        response = await stub.SubmitAndWaitForTransactionTree(request)

        return await self._codec.decode_exercise_response(response.transaction)

    async def exercise_by_key(
        self,
        __template_id: Union[str, TypeConName],
        __choice_name: str,
        __key: Any,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> "ExerciseResponse":
        """
        Exercise a choice on a contract identified by its contract key.

        :param __template_id:
            The template of the contract to be created (positional argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __choice_name:
            The name of the choice to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __key:
            The key of the contract to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __argument:
            The choice arguments. Can be omitted for choices that take no argument.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        stub = CommandServiceStub(self.channel)

        commands = [
            G_Command(
                exerciseByKey=await self._codec.encode_exercise_by_key_command(
                    __template_id, __choice_name, __key, __argument
                )
            )
        ]
        request = self._submit_and_wait_request(commands, workflow_id, command_id)
        response = await stub.SubmitAndWaitForTransactionTree(request)

        return await self._codec.decode_exercise_response(response.transaction)

    async def archive(
        self,
        __contract_id: ContractId,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ArchiveEvent:
        """
        Archive a choice on a contract identified by its contract ID.

        :param __contract_id:
            The contract ID of the contract to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        await self.exercise(
            __contract_id, "Archive", workflow_id=workflow_id, command_id=command_id
        )
        return ArchiveEvent(__contract_id)

    async def archive_by_key(
        self,
        __template_id: str,
        __key: Any,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ArchiveEvent:
        """
        Exercise a choice on a contract identified by its contract key.

        :param __template_id:
            The template of the contract to be created (positional argument only).

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param __key:
            The key of the contract to exercise.

            Note that future versions of dazl reserve the right to rename this parameter name at any
            time; it should be passed in as a positional parameter and never by name.
        :param workflow_id:
            An optional workflow ID.
        :param command_id:
            An optional command ID. If unspecified, a random one will be created.
        :return:
            The return value of the choice, together with a list of events that occurred as a result
            of exercising the choice.
        """
        response = await self.exercise_by_key(
            __template_id, "Archive", __key, workflow_id=workflow_id, command_id=command_id
        )
        return next(iter(event for event in response.events if isinstance(event, ArchiveEvent)))

    def _ensure_act_as(self) -> Party:
        act_as_party = next(iter(self._config.access.act_as), None)
        if not act_as_party:
            raise ValueError("current access rights do not include any act-as parties")
        return act_as_party

    @staticmethod
    def _workflow_id(workflow_id: Optional[str]) -> Optional[str]:
        if workflow_id:
            if not LEDGER_STRING_REGEX.match(workflow_id):
                raise ValueError("workflow_id must be a valid ledger string")
            return workflow_id
        else:
            return None

    @staticmethod
    def _command_id(command_id: Optional[str]) -> str:
        if command_id:
            if not LEDGER_STRING_REGEX.match(command_id):
                raise ValueError("command_id must be a valid ledger string")
            return command_id
        else:
            return uuid.uuid4().hex

    def _submit_and_wait_request(
        self,
        commands: Collection[G_Command],
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> G_SubmitAndWaitRequest:
        return G_SubmitAndWaitRequest(
            commands=G_Commands(
                ledger_id=self._config.access.ledger_id,
                application_id=self._config.access.application_name,
                command_id=self._command_id(command_id),
                workflow_id=self._workflow_id(workflow_id),
                party=self._ensure_act_as(),
                commands=commands,
                act_as=self._config.access.act_as,
                read_as=self._config.access.read_only_as,
            )
        )

    # endregion

    # region Read API

    def query(self, __template_id: str = "*", __query: Query = None) -> "QueryStream":
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        :param __template_id:
            The name of the template for which to fetch contracts.
        :param __query:
            A filter to apply to the set of returned contracts.
        """
        return QueryStream(
            self, parse_query({__template_id: __query}, server_side_filters=False), UNTIL_END
        )

    def query_many(self, *queries: Queries) -> "QueryStream":
        """
        Return the create events from the active contract set service as a stream.

        If you find yourself repeatedly calling :meth:`query` or :meth:`query_many` over the same
        set of templates, you may want to consider :class:`ACS` instead, which is a utility class
        that helps you maintain a "live" state of the ACS.

        :param queries:
            A map of template IDs to filter to apply to the set of returned contracts.
        """
        return QueryStream(self, parse_query(*queries, server_side_filters=False), UNTIL_END)

    def stream(
        self, __template_id: str = "*", __query: Query = None, *, offset: Optional[str] = None
    ) -> "QueryStream":
        """
        Stream create/archive events.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        :param __template_id:
            The name of the template for which to fetch contracts.
        :param __query:
            A filter to apply to the set of returned contracts. Note that this does not filter
            :class:`ArchiveEvent`; readers of the stream MUST be able to cope with "mismatched"
            archives that come from the result of applying a filter.
        :param offset:
            An optional offset at which to start receiving events. If ``None``, start from the
            beginning.
        """
        return QueryStream(
            self,
            parse_query({__template_id: __query}, server_side_filters=False),
            from_offset_until_forever(offset),
        )

    def stream_many(self, *queries: Queries, offset: Optional[str] = None) -> "QueryStream":
        """
        Stream create/archive events from more than one template ID in the same stream.

        When ``offset`` is ``None``, create events from the active contract set are returned first,
        followed by a continuous stream of updates (creates/archives).

        Otherwise, ``offset`` can be supplied to resume a stream from a prior point where a
        ``Boundary`` was returned from a previous stream.

        :param queries:
            A map of template IDs to filter to apply to the set of returned contracts. Note that
            this does not filter :class:`ArchiveEvent`; readers of the stream MUST be able to cope
            with "mismatched" archives that come from the result of applying a filter.
        :param offset:
            An optional offset at which to start receiving events. If ``None``, start from the
            beginning.
        """
        return QueryStream(
            self,
            parse_query(*queries, server_side_filters=False),
            from_offset_until_forever(offset),
        )

    # endregion

    # region Party Management calls

    async def allocate_party(
        self, *, identifier_hint: str = None, display_name: str = None
    ) -> "PartyInfo":
        """
        Allocate a new party.
        """
        stub = PartyManagementServiceStub(self.channel)
        request = G_AllocatePartyRequest(
            party_id_hint=Party(identifier_hint) if identifier_hint else None,
            display_name=display_name,
        )
        response = await stub.AllocateParty(request)
        return Codec.decode_party_info(response.party_details)

    async def list_known_parties(self) -> Sequence[PartyInfo]:
        stub = PartyManagementServiceStub(self.channel)
        request = G_ListKnownPartiesRequest()
        response = await stub.ListKnownParties(request)
        return [Codec.decode_party_info(pd) for pd in response.party_details]

    # endregion

    # region Package Management calls

    async def get_package(self, package_id: PackageRef) -> bytes:
        stub = PackageServiceStub(self.channel)
        request = G_GetPackageRequest(
            ledger_id=self._config.access.ledger_id, package_id=package_id
        )
        response = await stub.GetPackage(request)
        return response.archive_payload

    async def list_package_ids(self) -> AbstractSet[PackageRef]:
        stub = PackageServiceStub(self.channel)
        request = G_ListPackagesRequest(ledger_id=self._config.access.ledger_id)
        response = await stub.ListPackages(request)
        return frozenset({PackageRef(pkg_id) for pkg_id in response.package_ids})

    async def upload_package(self, contents: bytes) -> None:
        stub = PackageManagementServiceStub(self.channel)
        request = G_UploadDarFileRequest(dar_file=contents)
        await stub.UploadDarFile(request)
        return

    # endregion


class QueryStream(aio.QueryStreamBase):
    def __init__(
        self,
        conn: Connection,
        filters: Optional[Mapping[TypeConName, Filter]],
        offset_range: LedgerOffsetRange,
    ):
        self.conn = conn
        self._filters = filters
        self._offset_range = offset_range
        self._response_stream = None  # type: Optional[UnaryStreamCall]

    async def close(self) -> None:
        if self._response_stream is not None:
            self._response_stream.cancel()
            self._response_stream = None

    async def items(self):
        """
        Return an asynchronous stream of events.

        .. code-block:: python

            async with conn.query('SampleApp:Iou') as query:
                async for r in query:
                    print(f"Offset: {r.offset}")
                    for event in r.events:
                        print(f"  Event: {event}")

        :return:
            A stream of responses, where each response contains one or more events at a particular
            offset.

            At least one initial :class:`Boundary` is always returned, even if the stream is empty.
            In this case, the first returned object is a :class:`Boundary` with ``offset=None``.
        """
        log = self.conn.config.logger

        filters = await self.conn.codec.encode_filters(self._filters)
        filters_by_party = {party: filters for party in self.conn.config.access.read_as}
        tx_filter_pb = G_TransactionFilter(filters_by_party=filters_by_party)

        try:
            offset = None
            if self._offset_range.begin is None:
                # when starting from the beginning of the ledger, the Active Contract Set service
                # lets us catch up more quickly than having to parse every create/archive event
                # ourselves
                log.debug("Reading from the ACS...")
                async for event in self._acs_events(tx_filter_pb):
                    if isinstance(event, CreateEvent):
                        await self._emit_create(event)
                    elif isinstance(event, Boundary):
                        offset = event.offset
                        await self._emit_boundary(event)
                    else:
                        warnings.warn(f"Received an unknown event: {event}", ProtocolWarning)
                    yield event
            else:
                log.debug(
                    "Skipped reading from the ACS because begin offset is %r",
                    self._offset_range.begin,
                )

            if self._offset_range != UNTIL_END:
                # now start returning events as they come off the transaction stream; note this
                # stream will never naturally close, so it's on the caller to call close() or to
                # otherwise exit our current context
                log.debug("Reading a transaction stream: %s", self._offset_range)
                async for event in self._tx_events(tx_filter_pb, offset):
                    if isinstance(event, CreateEvent):
                        await self._emit_create(event)
                    elif isinstance(event, ArchiveEvent):
                        await self._emit_archive(event)
                    elif isinstance(event, Boundary):
                        await self._emit_boundary(event)
                    else:
                        warnings.warn(f"Received an unknown event: {event}", ProtocolWarning)
                    yield event
            else:
                log.debug(
                    "Not reading from transaction stream because we were only asked for a snapshot."
                )
        finally:
            await self.close()

    async def _acs_events(
        self, filter_pb: G_TransactionFilter
    ) -> AsyncIterable[Union[CreateEvent, Boundary]]:
        stub = ActiveContractsServiceStub(self.conn.channel)

        request = G_GetActiveContractsRequest(
            ledger_id=self.conn.config.access.ledger_id, filter=filter_pb
        )
        self._response_stream = response_stream = stub.GetActiveContracts(request)

        offset = None
        async for response in response_stream:
            for event in response.active_contracts:
                c_evt = await self.conn.codec.decode_created_event(event)
                if self._is_match(c_evt):
                    yield c_evt
            # for ActiveContractSetResponse messages, only the last offset is actually relevant
            offset = response.offset
        yield Boundary(offset)

    async def _tx_events(
        self, filter_pb: G_TransactionFilter, begin_offset: Optional[str]
    ) -> AsyncIterable[Union[CreateEvent, ArchiveEvent, Boundary]]:
        stub = TransactionServiceStub(self.conn.channel)

        request = G_GetTransactionsRequest(
            ledger_id=self.conn.config.access.ledger_id,
            filter=filter_pb,
            begin=self.conn.codec.encode_begin_offset(begin_offset),
        )

        self._response_stream = response_stream = stub.GetTransactions(request)
        async for response in response_stream:
            for tx in response.transactions:
                for event in tx.events:
                    event_type = event.WhichOneof("event")
                    if event_type == "created":
                        c_evt = await self.conn.codec.decode_created_event(event.created)
                        if self._is_match(c_evt):
                            yield c_evt
                    elif event_type == "archived":
                        yield await self.conn.codec.decode_archived_event(event.archived)
                    else:
                        warnings.warn(f"Unknown Event({event_type}=...)", ProtocolWarning)
                yield Boundary(tx.offset)

    def _is_match(self, event: CreateEvent) -> bool:
        # if there are no filters, then everything is a match
        if self._filters is None:
            return True

        for name, f in self._filters.items():
            # for each filter, if this contract type could be interpreted as a match for the current
            # filter, then apply a client-side filter
            if is_match(name, event.contract_id.value_type):
                return f.client_side is None or f.client_side(event.payload)

        # we checked every name, but none of them matched
        return False



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/testing/reset_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/testing/reset_service.proto",
    package="com.daml.ledger.api.v1.testing",
    syntax="proto3",
    serialized_options=b"\n\036com.daml.ledger.api.v1.testingB\026ResetServiceOuterClass\252\002\036Com.Daml.Ledger.Api.V1.Testing",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n2com/daml/ledger/api/v1/testing/reset_service.proto\x12\x1e\x63om.daml.ledger.api.v1.testing\x1a\x1bgoogle/protobuf/empty.proto"!\n\x0cResetRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t2]\n\x0cResetService\x12M\n\x05Reset\x12,.com.daml.ledger.api.v1.testing.ResetRequest\x1a\x16.google.protobuf.EmptyBY\n\x1e\x63om.daml.ledger.api.v1.testingB\x16ResetServiceOuterClass\xaa\x02\x1e\x43om.Daml.Ledger.Api.V1.Testingb\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,
    ],
)


_RESETREQUEST = _descriptor.Descriptor(
    name="ResetRequest",
    full_name="com.daml.ledger.api.v1.testing.ResetRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.testing.ResetRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=115,
    serialized_end=148,
)

DESCRIPTOR.message_types_by_name["ResetRequest"] = _RESETREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ResetRequest = _reflection.GeneratedProtocolMessageType(
    "ResetRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _RESETREQUEST,
        "__module__": "com.daml.ledger.api.v1.testing.reset_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.ResetRequest)
    },
)
_sym_db.RegisterMessage(ResetRequest)


DESCRIPTOR._options = None

_RESETSERVICE = _descriptor.ServiceDescriptor(
    name="ResetService",
    full_name="com.daml.ledger.api.v1.testing.ResetService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=150,
    serialized_end=243,
    methods=[
        _descriptor.MethodDescriptor(
            name="Reset",
            full_name="com.daml.ledger.api.v1.testing.ResetService.Reset",
            index=0,
            containing_service=None,
            input_type=_RESETREQUEST,
            output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_RESETSERVICE)

DESCRIPTOR.services_by_name["ResetService"] = _RESETSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/testing/time_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import (
    empty_pb2 as google_dot_protobuf_dot_empty__pb2,
    timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/testing/time_service.proto",
    package="com.daml.ledger.api.v1.testing",
    syntax="proto3",
    serialized_options=b"\n\036com.daml.ledger.api.v1.testingB\025TimeServiceOuterClass\252\002\036Com.Daml.Ledger.Api.V1.Testing",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n1com/daml/ledger/api/v1/testing/time_service.proto\x12\x1e\x63om.daml.ledger.api.v1.testing\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto"#\n\x0eGetTimeRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t"C\n\x0fGetTimeResponse\x12\x30\n\x0c\x63urrent_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp"\x83\x01\n\x0eSetTimeRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x30\n\x0c\x63urrent_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08new_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp2\xce\x01\n\x0bTimeService\x12l\n\x07GetTime\x12..com.daml.ledger.api.v1.testing.GetTimeRequest\x1a/.com.daml.ledger.api.v1.testing.GetTimeResponse0\x01\x12Q\n\x07SetTime\x12..com.daml.ledger.api.v1.testing.SetTimeRequest\x1a\x16.google.protobuf.EmptyBX\n\x1e\x63om.daml.ledger.api.v1.testingB\x15TimeServiceOuterClass\xaa\x02\x1e\x43om.Daml.Ledger.Api.V1.Testingb\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
    ],
)


_GETTIMEREQUEST = _descriptor.Descriptor(
    name="GetTimeRequest",
    full_name="com.daml.ledger.api.v1.testing.GetTimeRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.testing.GetTimeRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=147,
    serialized_end=182,
)


_GETTIMERESPONSE = _descriptor.Descriptor(
    name="GetTimeResponse",
    full_name="com.daml.ledger.api.v1.testing.GetTimeResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="current_time",
            full_name="com.daml.ledger.api.v1.testing.GetTimeResponse.current_time",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=184,
    serialized_end=251,
)


_SETTIMEREQUEST = _descriptor.Descriptor(
    name="SetTimeRequest",
    full_name="com.daml.ledger.api.v1.testing.SetTimeRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.testing.SetTimeRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="current_time",
            full_name="com.daml.ledger.api.v1.testing.SetTimeRequest.current_time",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="new_time",
            full_name="com.daml.ledger.api.v1.testing.SetTimeRequest.new_time",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=254,
    serialized_end=385,
)

_GETTIMERESPONSE.fields_by_name[
    "current_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEREQUEST.fields_by_name[
    "current_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEREQUEST.fields_by_name[
    "new_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name["GetTimeRequest"] = _GETTIMEREQUEST
DESCRIPTOR.message_types_by_name["GetTimeResponse"] = _GETTIMERESPONSE
DESCRIPTOR.message_types_by_name["SetTimeRequest"] = _SETTIMEREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTimeRequest = _reflection.GeneratedProtocolMessageType(
    "GetTimeRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTIMEREQUEST,
        "__module__": "com.daml.ledger.api.v1.testing.time_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.GetTimeRequest)
    },
)
_sym_db.RegisterMessage(GetTimeRequest)

GetTimeResponse = _reflection.GeneratedProtocolMessageType(
    "GetTimeResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTIMERESPONSE,
        "__module__": "com.daml.ledger.api.v1.testing.time_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.GetTimeResponse)
    },
)
_sym_db.RegisterMessage(GetTimeResponse)

SetTimeRequest = _reflection.GeneratedProtocolMessageType(
    "SetTimeRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _SETTIMEREQUEST,
        "__module__": "com.daml.ledger.api.v1.testing.time_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.testing.SetTimeRequest)
    },
)
_sym_db.RegisterMessage(SetTimeRequest)


DESCRIPTOR._options = None

_TIMESERVICE = _descriptor.ServiceDescriptor(
    name="TimeService",
    full_name="com.daml.ledger.api.v1.testing.TimeService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=388,
    serialized_end=594,
    methods=[
        _descriptor.MethodDescriptor(
            name="GetTime",
            full_name="com.daml.ledger.api.v1.testing.TimeService.GetTime",
            index=0,
            containing_service=None,
            input_type=_GETTIMEREQUEST,
            output_type=_GETTIMERESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="SetTime",
            full_name="com.daml.ledger.api.v1.testing.TimeService.SetTime",
            index=1,
            containing_service=None,
            input_type=_SETTIMEREQUEST,
            output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_TIMESERVICE)

DESCRIPTOR.services_by_name["TimeService"] = _TIMESERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from .model import Formatter

__all__ = ["DEFAULT_FORMATTER_NAME", "get_formatter"]

DEFAULT_FORMATTER_NAME = "pretty"


def get_formatter(fmt) -> "Formatter":
    """
    Return a formatter of the specified name.
    """
    if fmt is None:
        fmt = DEFAULT_FORMATTER_NAME

    if fmt == "json":
        from .fmt_json import JsonFormatter

        return JsonFormatter()
    elif fmt == "pretty":
        from .fmt_pretty import PrettyFormatter

        return PrettyFormatter()
    else:
        raise ValueError("unknown format: {}".format(fmt))



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module has been relocated to ``dazl.damlast.lookup``.
"""

from typing import Iterator, Tuple
import warnings

from ..damlast.daml_lf_1 import PackageRef
from ..damlast.lookup import validate_template as _validate_template

__all__ = ["validate_template", "template_reverse_globs"]

warnings.warn(
    "dazl.model.lookup is deprecated; use dazl.damlast.lookup instead.",
    DeprecationWarning,
    stacklevel=2,
)


def validate_template(template) -> "Tuple[PackageRef, str]":
    """
    Return a module and type name component from something that can be interpreted as a template.

    This function is deprecated and will be removed in dazl v8; please use
    ``dazl.damlast.lookup.validate_template`` instead.

    :param template:
        Any object that can be interpreted as an identifier for a template.
    """
    warnings.warn(
        "dazl.model.lookup.validate_template is deprecated and will be removed in dazl v8; "
        "use dazl.damlast.lookup.validate_template instead",
        DeprecationWarning,
        stacklevel=2,
    )

    return _validate_template(template, allow_deprecated_identifiers=True)


def template_reverse_globs(primary_only: bool, package_id: str, type_name: str) -> "Iterator[str]":
    """
    Return an iterator over strings that glob to a specified type.
    """
    # noinspection PyProtectedMember
    from ..client.events import _template_reverse_globs

    warnings.warn(
        "template_reverse_globs is deprecated; use either "
        "dazl.damlast.lookup.matching_normalizations (for template_reverse_globs(False, ...)) or "
        "dazl.damlast.lookup.normalize(for template_reverse_globs(True, ...)). "
        "Note that the new functions do NOT support periods as a delimiter between "
        "module names and entity names; you MUST use a colon.",
        DeprecationWarning,
    )
    return _template_reverse_globs(primary_only, package_id, type_name)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# fmt: off

from typing import (
    AsyncIterable as __AsyncStream,
    Awaitable as __Awaitable,
    Iterable as __Stream,
    Union as __Union,
    overload,
)

from grpc import Channel as __Channel
from grpc.aio import Channel as __AsyncChannel

from .config_management_service_pb2 import (
    GetTimeModelRequest,
    GetTimeModelResponse,
    SetTimeModelRequest,
    SetTimeModelResponse,
)

__all__ = [
    "ConfigManagementServiceStub",
]

class ConfigManagementServiceStub:
    @classmethod
    @overload
    def __new__(cls, channel: __Channel) -> _ConfigManagementServiceStub: ...
    @classmethod
    @overload
    def __new__(cls, channel: __AsyncChannel) -> _ConfigManagementServiceStub_Async: ...
    def GetTimeModel(self, __1: GetTimeModelRequest) -> __Union[GetTimeModelResponse, __Awaitable[GetTimeModelResponse]]: ...
    def SetTimeModel(self, __1: SetTimeModelRequest) -> __Union[SetTimeModelResponse, __Awaitable[SetTimeModelResponse]]: ...

class _ConfigManagementServiceStub(ConfigManagementServiceStub):
    def GetTimeModel(self, __1: GetTimeModelRequest) -> GetTimeModelResponse: ...
    def SetTimeModel(self, __1: SetTimeModelRequest) -> SetTimeModelResponse: ...

class _ConfigManagementServiceStub_Async(ConfigManagementServiceStub):
    def GetTimeModel(self, __1: GetTimeModelRequest) -> __Awaitable[GetTimeModelResponse]: ...
    def SetTimeModel(self, __1: SetTimeModelRequest) -> SetTimeModelResponse: ...



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/package_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import trace_context_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/package_service.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\030PackageServiceOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n,com/daml/ledger/api/v1/package_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/trace_context.proto"f\n\x13ListPackagesRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"+\n\x14ListPackagesResponse\x12\x13\n\x0bpackage_ids\x18\x01 \x03(\t"x\n\x11GetPackageRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x12\n\npackage_id\x18\x02 \x01(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"x\n\x12GetPackageResponse\x12;\n\rhash_function\x18\x01 \x01(\x0e\x32$.com.daml.ledger.api.v1.HashFunction\x12\x17\n\x0f\x61rchive_payload\x18\x02 \x01(\x0c\x12\x0c\n\x04hash\x18\x03 \x01(\t"~\n\x17GetPackageStatusRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x12\n\npackage_id\x18\x02 \x01(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"Y\n\x18GetPackageStatusResponse\x12=\n\x0epackage_status\x18\x01 \x01(\x0e\x32%.com.daml.ledger.api.v1.PackageStatus*,\n\rPackageStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0e\n\nREGISTERED\x10\x01*\x1a\n\x0cHashFunction\x12\n\n\x06SHA256\x10\x00\x32\xd7\x02\n\x0ePackageService\x12i\n\x0cListPackages\x12+.com.daml.ledger.api.v1.ListPackagesRequest\x1a,.com.daml.ledger.api.v1.ListPackagesResponse\x12\x63\n\nGetPackage\x12).com.daml.ledger.api.v1.GetPackageRequest\x1a*.com.daml.ledger.api.v1.GetPackageResponse\x12u\n\x10GetPackageStatus\x12/.com.daml.ledger.api.v1.GetPackageStatusRequest\x1a\x30.com.daml.ledger.api.v1.GetPackageStatusResponseBK\n\x16\x63om.daml.ledger.api.v1B\x18PackageServiceOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2.DESCRIPTOR,
    ],
)

_PACKAGESTATUS = _descriptor.EnumDescriptor(
    name="PackageStatus",
    full_name="com.daml.ledger.api.v1.PackageStatus",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="UNKNOWN",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="REGISTERED",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=728,
    serialized_end=772,
)
_sym_db.RegisterEnumDescriptor(_PACKAGESTATUS)

PackageStatus = enum_type_wrapper.EnumTypeWrapper(_PACKAGESTATUS)
_HASHFUNCTION = _descriptor.EnumDescriptor(
    name="HashFunction",
    full_name="com.daml.ledger.api.v1.HashFunction",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="SHA256",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=774,
    serialized_end=800,
)
_sym_db.RegisterEnumDescriptor(_HASHFUNCTION)

HashFunction = enum_type_wrapper.EnumTypeWrapper(_HASHFUNCTION)
UNKNOWN = 0
REGISTERED = 1
SHA256 = 0


_LISTPACKAGESREQUEST = _descriptor.Descriptor(
    name="ListPackagesRequest",
    full_name="com.daml.ledger.api.v1.ListPackagesRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.ListPackagesRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.ListPackagesRequest.trace_context",
            index=1,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=116,
    serialized_end=218,
)


_LISTPACKAGESRESPONSE = _descriptor.Descriptor(
    name="ListPackagesResponse",
    full_name="com.daml.ledger.api.v1.ListPackagesResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="package_ids",
            full_name="com.daml.ledger.api.v1.ListPackagesResponse.package_ids",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=220,
    serialized_end=263,
)


_GETPACKAGEREQUEST = _descriptor.Descriptor(
    name="GetPackageRequest",
    full_name="com.daml.ledger.api.v1.GetPackageRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetPackageRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="package_id",
            full_name="com.daml.ledger.api.v1.GetPackageRequest.package_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetPackageRequest.trace_context",
            index=2,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=265,
    serialized_end=385,
)


_GETPACKAGERESPONSE = _descriptor.Descriptor(
    name="GetPackageResponse",
    full_name="com.daml.ledger.api.v1.GetPackageResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="hash_function",
            full_name="com.daml.ledger.api.v1.GetPackageResponse.hash_function",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="archive_payload",
            full_name="com.daml.ledger.api.v1.GetPackageResponse.archive_payload",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="hash",
            full_name="com.daml.ledger.api.v1.GetPackageResponse.hash",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=387,
    serialized_end=507,
)


_GETPACKAGESTATUSREQUEST = _descriptor.Descriptor(
    name="GetPackageStatusRequest",
    full_name="com.daml.ledger.api.v1.GetPackageStatusRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetPackageStatusRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="package_id",
            full_name="com.daml.ledger.api.v1.GetPackageStatusRequest.package_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetPackageStatusRequest.trace_context",
            index=2,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=509,
    serialized_end=635,
)


_GETPACKAGESTATUSRESPONSE = _descriptor.Descriptor(
    name="GetPackageStatusResponse",
    full_name="com.daml.ledger.api.v1.GetPackageStatusResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="package_status",
            full_name="com.daml.ledger.api.v1.GetPackageStatusResponse.package_status",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=637,
    serialized_end=726,
)

_LISTPACKAGESREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETPACKAGEREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETPACKAGERESPONSE.fields_by_name["hash_function"].enum_type = _HASHFUNCTION
_GETPACKAGESTATUSREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETPACKAGESTATUSRESPONSE.fields_by_name["package_status"].enum_type = _PACKAGESTATUS
DESCRIPTOR.message_types_by_name["ListPackagesRequest"] = _LISTPACKAGESREQUEST
DESCRIPTOR.message_types_by_name["ListPackagesResponse"] = _LISTPACKAGESRESPONSE
DESCRIPTOR.message_types_by_name["GetPackageRequest"] = _GETPACKAGEREQUEST
DESCRIPTOR.message_types_by_name["GetPackageResponse"] = _GETPACKAGERESPONSE
DESCRIPTOR.message_types_by_name["GetPackageStatusRequest"] = _GETPACKAGESTATUSREQUEST
DESCRIPTOR.message_types_by_name["GetPackageStatusResponse"] = _GETPACKAGESTATUSRESPONSE
DESCRIPTOR.enum_types_by_name["PackageStatus"] = _PACKAGESTATUS
DESCRIPTOR.enum_types_by_name["HashFunction"] = _HASHFUNCTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ListPackagesRequest = _reflection.GeneratedProtocolMessageType(
    "ListPackagesRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTPACKAGESREQUEST,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ListPackagesRequest)
    },
)
_sym_db.RegisterMessage(ListPackagesRequest)

ListPackagesResponse = _reflection.GeneratedProtocolMessageType(
    "ListPackagesResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTPACKAGESRESPONSE,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.ListPackagesResponse)
    },
)
_sym_db.RegisterMessage(ListPackagesResponse)

GetPackageRequest = _reflection.GeneratedProtocolMessageType(
    "GetPackageRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETPACKAGEREQUEST,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetPackageRequest)
    },
)
_sym_db.RegisterMessage(GetPackageRequest)

GetPackageResponse = _reflection.GeneratedProtocolMessageType(
    "GetPackageResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETPACKAGERESPONSE,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetPackageResponse)
    },
)
_sym_db.RegisterMessage(GetPackageResponse)

GetPackageStatusRequest = _reflection.GeneratedProtocolMessageType(
    "GetPackageStatusRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETPACKAGESTATUSREQUEST,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetPackageStatusRequest)
    },
)
_sym_db.RegisterMessage(GetPackageStatusRequest)

GetPackageStatusResponse = _reflection.GeneratedProtocolMessageType(
    "GetPackageStatusResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETPACKAGESTATUSRESPONSE,
        "__module__": "com.daml.ledger.api.v1.package_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetPackageStatusResponse)
    },
)
_sym_db.RegisterMessage(GetPackageStatusResponse)


DESCRIPTOR._options = None

_PACKAGESERVICE = _descriptor.ServiceDescriptor(
    name="PackageService",
    full_name="com.daml.ledger.api.v1.PackageService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=803,
    serialized_end=1146,
    methods=[
        _descriptor.MethodDescriptor(
            name="ListPackages",
            full_name="com.daml.ledger.api.v1.PackageService.ListPackages",
            index=0,
            containing_service=None,
            input_type=_LISTPACKAGESREQUEST,
            output_type=_LISTPACKAGESRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetPackage",
            full_name="com.daml.ledger.api.v1.PackageService.GetPackage",
            index=1,
            containing_service=None,
            input_type=_GETPACKAGEREQUEST,
            output_type=_GETPACKAGERESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetPackageStatus",
            full_name="com.daml.ledger.api.v1.PackageService.GetPackageStatus",
            index=2,
            containing_service=None,
            input_type=_GETPACKAGESTATUSREQUEST,
            output_type=_GETPACKAGESTATUSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_PACKAGESERVICE)

DESCRIPTOR.services_by_name["PackageService"] = _PACKAGESERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'python'}

packages = \
['dazl',
 'dazl._gen',
 'dazl._gen.com',
 'dazl._gen.com.daml',
 'dazl._gen.com.daml.daml_lf_dev',
 'dazl._gen.com.daml.ledger',
 'dazl._gen.com.daml.ledger.api',
 'dazl._gen.com.daml.ledger.api.v1',
 'dazl._gen.com.daml.ledger.api.v1.admin',
 'dazl._gen.com.daml.ledger.api.v1.testing',
 'dazl._gen.google',
 'dazl._gen.google.rpc',
 'dazl.cli',
 'dazl.client',
 'dazl.damlast',
 'dazl.ledger',
 'dazl.ledger.aio',
 'dazl.ledger.blocking',
 'dazl.ledger.config',
 'dazl.ledger.grpc',
 'dazl.ledgerutil',
 'dazl.metrics',
 'dazl.model',
 'dazl.pretty',
 'dazl.pretty.table',
 'dazl.prim',
 'dazl.protocols',
 'dazl.protocols.v0',
 'dazl.protocols.v1',
 'dazl.protocols.v1.model',
 'dazl.query',
 'dazl.scheduler',
 'dazl.server',
 'dazl.util',
 'dazl.values']

package_data = \
{'': ['*']}

install_requires = \
['grpcio>=1.32.0', 'protobuf>=3.12.0', 'requests', 'semver', 'toposort']

extras_require = \
{':python_version < "3.8.0"': ['typing_extensions'],
 ':python_version >= "3.6.0" and python_version < "3.7.0"': ['dataclasses'],
 'oauth': ['google-auth', 'oauthlib'],
 'prometheus': ['prometheus_client'],
 'pygments': ['pygments'],
 'server': ['aiohttp']}

entry_points = \
{'console_scripts': ['dazl = dazl.cli:main']}

setup_kwargs = {
    'name': 'dazl',
    'version': '7.5.2a1',
    'description': 'high-level Ledger API client for DAML ledgers',
    'long_description': 'dazl\n====\n\n[![License](https://img.shields.io/badge/License-Apache%202.0-blue.svg)](https://github.com/digital-asset/dazl-client/blob/master/LICENSE)\n<a href="https://circleci.com/gh/digital-asset/dazl-client">\n<img src="https://circleci.com/gh/digital-asset/dazl-client.svg?style=svg">\n</a>\n\nCopyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All Rights Reserved.\nSPDX-License-Identifier: Apache-2.0\n\n\nRich Python bindings for accessing Ledger API-based applications.\n\nDocumentation\n-------------\nThe user documentation is available online [here](https://digital-asset.github.io/dazl-client).\n\nInstallation\n------------\nIf you just want to use the library, you can install it locally with `pip`:\n```sh\npip install --user dazl\n```\n\nRequirements\n------------\n* Python 3.6+\n* [Daml Connect](https://www.daml.com)\n* Python gRPC libraries (1.32.0 or later) and Protobuf\n\n**WARNING:** The next major version of dazl (v8.0.0) will require **Python 3.7** or later.\n\nExamples\n--------\n\nAll of the examples below assume you imported `dazl`, and are running a ledger with the default scenario generated with `daml new`.\n\nConnect to the ledger and submit a single command:\n\n```py\nimport asyncio\nimport dazl\n\nasync def main():\n    async with dazl.connect(\'http://localhost:6865\', act_as=\'Alice\') as client:\n        contract = { \'issuer\' : \'Alice\', \'owner\' : \'Alice\', \'name\' : \'hello world!\' }\n        await client.create(\'Main:Asset\', contract)\n\n# Python 3.7+\nasyncio.run(main())\n\n# Python 3.6+\nloop = asyncio.get_event_loop()\nloop.run_until_complete(main())\n```\n\nConnect to the ledger as a single party, print all contracts, and close:\n\n```py\nimport asyncio\nimport dazl\nfrom dazl.ledgerutil import ACS\n\nasync def main():\n    async with dazl.connect(\'http://localhost:6865\', read_as=\'Alice\') as conn:\n        async with ACS(conn, {"*": {}}) as acs:\n            snapshot = await acs.read()\n            \n    print(snapshot)\n\n# Python 3.7+\nasyncio.run(main())\n\n# Python 3.6+\nloop = asyncio.get_event_loop()\nloop.run_until_complete(main())\n```\n\nBuilding locally\n----------------\n\nYou will need additional dependencies to build locally:\n\n* GNU Make\n* [Poetry](https://python-poetry.org/) for build/dependency management\n\nOnce you have these prerequisites in place:\n\n```sh\nmake build\n```\n\nIf you see errors about incompatible python versions, switch your environment to python3 using `poetry env use python3`, for instance.\n\nTests\n-----\n\nTests in dazl are written using [pytest](https://docs.pytest.org/en/latest/). You can run them by doing:\n\n```sh\nmake test\n```\n',
    'author': 'Davin K. Tanabe',
    'author_email': 'davin.tanabe@digitalasset.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/digital-asset/dazl-client',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
import abc
import sys
from typing import (
    AbstractSet,
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    DefaultDict,
    List,
    Optional,
    Sequence,
    TypeVar,
    Union,
    overload,
)

from .. import (
    Connection as _Connection,
    PackageService as _PackageService,
    QueryStream as _QueryStream,
)
from ...damlast.daml_lf_1 import PackageRef, TypeConName
from ...prim import ContractData, ContractId
from ...query import Queries, Query
from ..api_types import (
    ArchiveEvent,
    Boundary,
    Command,
    CreateEvent,
    ExerciseResponse,
    PartyInfo,
    SubmitResponse,
)
from .pkgloader import PackageLoader

if sys.version_info >= (3, 8):
    from typing import Protocol, runtime_checkable
else:
    from typing_extensions import Protocol, runtime_checkable

__all__ = ["PackageService", "Connection", "QueryStream", "QueryStreamBase", "PackageLoader"]

ConnSelf = TypeVar("ConnSelf", bound="Connection")
QSSelf = TypeVar("QSSelf", bound="QueryStream")
CreateFn = TypeVar("CreateFn", bound=Callable[[CreateEvent], SubmitResponse])
ACreateFn = TypeVar("ACreateFn", bound=Callable[[CreateEvent], Awaitable[SubmitResponse]])
ArchiveFn = TypeVar("ArchiveFn", bound=Callable[[ArchiveEvent], SubmitResponse])
AArchiveFn = TypeVar("AArchiveFn", bound=Callable[[ArchiveEvent], Awaitable[SubmitResponse]])
BoundaryFn = TypeVar("BoundaryFn", bound=Callable[[Boundary], SubmitResponse])
ABoundaryFn = TypeVar("ABoundaryFn", bound=Callable[[Boundary], Awaitable[SubmitResponse]])

# mypy treats Callables as _covariant_ in their arguments, and _contravariant_ in their return
# types. That means that A -> A and B -> B are incompatible types.
#
# However, we can define an object that has two overloaded __call__ signatures, which means that
# we have an *object* that can be used like a function, but supports adding additional "overloads",
# such that the base type supports being called with A, and returns A, and the subtype supports
# that, AS WELL AS being called with B and returning B.
#
# mypy completely understands what is going on here, and manages to validate correct usages of these
# decorators, *as well as* flagging incorrect usages!
class ACreateDecorator(Protocol):
    @overload
    def __call__(self, __fn: CreateFn) -> CreateFn: ...
    @overload
    def __call__(self, __fn: ACreateFn) -> ACreateFn: ...

class AArchiveDecorator(Protocol):
    @overload
    def __call__(self, __fn: ArchiveFn) -> ArchiveFn: ...
    @overload
    def __call__(self, __fn: AArchiveFn) -> AArchiveFn: ...

class ABoundaryDecorator(Protocol):
    @overload
    def __call__(self, __fn: BoundaryFn) -> BoundaryFn: ...
    @overload
    def __call__(self, __fn: ABoundaryFn) -> ABoundaryFn: ...

class PackageService(_PackageService, Protocol):
    async def get_package(self, __package_id: PackageRef) -> bytes: ...
    async def list_package_ids(self) -> AbstractSet[PackageRef]: ...

@runtime_checkable
class Connection(_Connection, PackageService, Protocol):
    async def __aenter__(self: ConnSelf) -> ConnSelf: ...
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None: ...
    async def open(self) -> None: ...
    async def close(self) -> None: ...
    async def submit(
        self,
        __commands: Union[Command, Sequence[Command]],
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> None: ...
    async def create(
        self,
        __template_id: Union[str, TypeConName],
        __payload: ContractData,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> CreateEvent: ...
    async def exercise(
        self,
        __contract_id: ContractId,
        __choice_name: str,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ExerciseResponse: ...
    async def create_and_exercise(
        self,
        __template_id: Union[str, TypeConName],
        __payload: ContractData,
        __choice_name: str,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ExerciseResponse: ...
    async def exercise_by_key(
        self,
        __template_id: Union[str, TypeConName],
        __choice_name: str,
        __key: Any,
        __argument: Optional[ContractData] = None,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ExerciseResponse: ...
    async def archive(
        self,
        __contract_id: ContractId,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ArchiveEvent: ...
    async def archive_by_key(
        self,
        __template_id: str,
        __key: Any,
        *,
        workflow_id: Optional[str] = None,
        command_id: Optional[str] = None,
    ) -> ArchiveEvent: ...
    def query(self, __template_id: str = "*", __query: Query = None) -> QueryStream: ...
    def query_many(self, *q: Queries) -> QueryStream: ...
    def stream(
        self, __template_id: str = "*", __query: Query = None, *, offset: Optional[str] = None
    ) -> QueryStream: ...
    def stream_many(self, *q: Queries, offset: Optional[str] = None) -> QueryStream: ...
    async def allocate_party(
        self, *, identifier_hint: Optional[str] = None, display_name: Optional[str] = None
    ) -> PartyInfo: ...
    async def list_known_parties(self) -> Sequence[PartyInfo]: ...
    async def upload_package(self, contents: bytes) -> None: ...

# PyCharm doesn't know what to make of these overloads with respect to the parent protocol,
# but mypy understands that these type signatures do not conflict with the parent base class
# noinspection PyProtocol,PyMethodOverriding
@runtime_checkable
class QueryStream(_QueryStream, Protocol):
    @overload
    def on_create(self) -> ACreateDecorator: ...
    @overload
    def on_create(self, __fn: CreateFn) -> CreateFn: ...
    @overload
    def on_create(self, __fn: ACreateFn) -> ACreateFn: ...
    @overload
    def on_create(self, __name: Union[str, TypeConName]) -> ACreateDecorator: ...
    @overload
    def on_create(self, __name: Union[str, TypeConName], __fn: CreateFn) -> CreateFn: ...
    @overload
    def on_create(self, __name: Union[str, TypeConName], __fn: ACreateFn) -> ACreateFn: ...
    @overload
    def on_archive(self) -> AArchiveDecorator: ...
    @overload
    def on_archive(self, __fn: ArchiveFn) -> ArchiveFn: ...
    @overload
    def on_archive(self, __fn: AArchiveFn) -> AArchiveFn: ...
    @overload
    def on_archive(self, __name: Union[str, TypeConName]) -> AArchiveDecorator: ...
    @overload
    def on_archive(self, __name: Union[str, TypeConName], __fn: ArchiveFn) -> ArchiveFn: ...
    @overload
    def on_archive(self, __name: Union[str, TypeConName], __fn: AArchiveFn) -> AArchiveFn: ...
    @overload
    def on_boundary(self) -> ABoundaryDecorator: ...
    @overload
    def on_boundary(self, __fn: BoundaryFn) -> BoundaryFn: ...
    @overload
    def on_boundary(self, __fn: ABoundaryFn) -> ABoundaryFn: ...
    def creates(self) -> AsyncIterator[CreateEvent]: ...
    def events(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent]]: ...
    def items(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]: ...
    def __aiter__(self) -> AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]: ...
    async def run(self) -> None: ...
    async def close(self) -> None: ...
    async def __aenter__(self: QSSelf) -> QSSelf: ...
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None: ...

class QueryStreamBase(QueryStream, abc.ABC):
    @property
    def _callbacks(self) -> DefaultDict[str, List[Callable]]: ...
    @abc.abstractmethod
    def items(self): ...
    async def _emit(self, name: str, obj: Any) -> None: ...
    async def _emit_create(self, event: CreateEvent) -> None: ...
    async def _emit_archive(self, event: ArchiveEvent) -> None: ...
    async def _emit_boundary(self, event: Boundary) -> None: ...



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
from contextlib import ExitStack
from os import PathLike, fspath
from pathlib import Path
import time
from typing import List, Sequence, Union
import warnings

from .. import LOG


class LocalDarRepository:
    def __init__(self):
        warnings.warn(
            "LocalDarRepository is deprecated; use PackageLookup instead",
            DeprecationWarning,
            stacklevel=2,
        )
        self._context = ExitStack()
        self._dar_paths = []  # type: List[Path]
        self._files = set()
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", DeprecationWarning)

            from ..model.types_store import PackageStore

            self.store = PackageStore.empty()

    def _add_source(self, path: Path) -> None:
        ext = path.suffix.lower()

        if ext == ".daml":
            LOG.error("Reading metadata directly from a DAML file is not supported.")
            raise ValueError("Unsupported extension: .daml")

        elif ext == ".dalf":
            LOG.error("Reading metadata directly from a DALF file is not supported.")
            raise ValueError("Unsupported extension: .dalf")

        elif ext == ".dar":
            dar_parse_start_time = time.time()
            with warnings.catch_warnings():
                warnings.simplefilter("ignore", DeprecationWarning)

                from ..util.dar import DarFile

                dar = self._context.enter_context(DarFile(path))
            dar_package = dar.read_metadata()
            self._dar_paths.append(path)
            self.store.register_all(dar_package)
            dar_parse_end_time = time.time()
            LOG.debug("Parsed a dar in %s seconds.", dar_parse_end_time - dar_parse_start_time)

        else:
            LOG.error("Unknown extension: %s", ext)
            raise ValueError(f"Unknown extension: {ext}")

    def add_source(self, *files: Union[str, PathLike, Path]) -> None:
        """
        Add a source file (either a .daml file, .dalf file, or a .dar file).

        Attempts to add the same file more than once will be ignored.

        :param files: Files to add to the archive.
        """
        for file in files:
            path = Path(fspath(file)).resolve(strict=True)
            if path not in self._files:
                self._files.add(path)
                self._add_source(path)

    def get_daml_archives(self) -> Sequence[Path]:
        return self._dar_paths

    def __enter__(self):
        """
        Does nothing.
        """
        self._context.__enter__()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Delete all managed resources in the reverse order that they were created.
        """
        self._context.__exit__(exc_type, exc_val, exc_tb)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import (
    package_management_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2,
)


class PackageManagementServiceStub(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Query the Daml-LF packages supported by the ledger participant and upload
    DAR files. We use 'backing participant' to refer to this specific participant
    in the methods of this API.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListKnownPackages = channel.unary_unary(
            "/com.daml.ledger.api.v1.admin.PackageManagementService/ListKnownPackages",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesResponse.FromString,
        )
        self.UploadDarFile = channel.unary_unary(
            "/com.daml.ledger.api.v1.admin.PackageManagementService/UploadDarFile",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileResponse.FromString,
        )


class PackageManagementServiceServicer(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Query the Daml-LF packages supported by the ledger participant and upload
    DAR files. We use 'backing participant' to refer to this specific participant
    in the methods of this API.
    """

    def ListKnownPackages(self, request, context):
        """Returns the details of all Daml-LF packages known to the backing participant.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UploadDarFile(self, request, context):
        """Upload a DAR file to the backing participant.
        Depending on the ledger implementation this might also make the package
        available on the whole ledger. This call might not be supported by some
        ledger implementations. Canton could be an example, where uploading a DAR
        is not sufficient to render it usable, it must be activated first.
        This call may:
        - Succeed, if the package was successfully uploaded, or if the same package
        was already uploaded before.
        - Respond with a gRPC error
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``UNIMPLEMENTED``: if DAR package uploading is not supported by the backing participant
        - ``INVALID_ARGUMENT``: if the DAR file is too big or malformed. The maximum supported size is implementation specific.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_PackageManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ListKnownPackages": grpc.unary_unary_rpc_method_handler(
            servicer.ListKnownPackages,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesResponse.SerializeToString,
        ),
        "UploadDarFile": grpc.unary_unary_rpc_method_handler(
            servicer.UploadDarFile,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "com.daml.ledger.api.v1.admin.PackageManagementService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class PackageManagementService(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Query the Daml-LF packages supported by the ledger participant and upload
    DAR files. We use 'backing participant' to refer to this specific participant
    in the methods of this API.
    """

    @staticmethod
    def ListKnownPackages(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/com.daml.ledger.api.v1.admin.PackageManagementService/ListKnownPackages",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.ListKnownPackagesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UploadDarFile(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/com.daml.ledger.api.v1.admin.PackageManagementService/UploadDarFile",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_package__management__service__pb2.UploadDarFileResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
